//===================================================================
// This file is provided as part of the ClearImage COM Development Kit
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT  
// WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,  
// INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES  
// OF MERCHANTABILITY AND/OR FITNESS FOR A  PARTICULAR PURPOSE. 
// 
// Copyright (C) 2003-2004.  Inlite Research, Inc. All rights reserved 
//===================================================================

#include "stdafx.h"
#include <stdio.h>
CComModule _Module; 

bool ReadMicr  (ICiImagePtr &Image, char *rot);
bool ReadOneMicr  (char *fileName);

//===============================================================
int main(int argc, char* argv[])
	{
	long nMasterId = 0;
	if (argc > 1)
		nMasterId = atol (argv[1]);
		//  Initailzie COM
	CoInitialize(NULL);
		//  Create External object]
	try 
		{
		char s[256];
		ICiServerPtr Ci;
			// Create server
		HRESULT hr = Ci.CreateInstance(__uuidof(CiServer));
		if (FAILED(hr)) _com_issue_error (hr);
			//  Open License
		if (nMasterId > 0)
			Ci->OpenExt ((long) GetModuleHandle(NULL), nMasterId, 0);
			//  Display server status info
		cout << "ClearImage COM Version  " 
				 << Ci->VerMajor << "." << Ci->VerMinor << "." << Ci->VerRelease << endl;
		cout << Ci->GetInfo(ciModulesList, 0) << endl;
			//  Process files
		for( ; ; )
			{
			cout << "Enter \"q\" to quit, or file name: ";
			cin.getline (s, sizeof(s));
			if(!stricmp(s, "q"))
					break;
			if (strlen(s) == 0)
				continue;
				// Delete quotes ("xxxx")
			if ((strlen(s) > 2)  && (*s == '\"') && (*(s + strlen(s) - 1) == '\"'))
				{
				strcpy (s, s+1);
				*(s + strlen(s) - 1) = 0;
				}
			ReadOneMicr(s);		//  Simple MICR reading code

			//  Read PDF417 and DataMatrix with common CiImage
			ICiImagePtr Image = Ci->CreateImage();
			Image->Open (_bstr_t(s), 1);
			Image->RotateLeft();
			if (!ReadMicr(Image, "left"))
				{
				Image->Flip();
				if (ReadMicr(Image, "right"))
					Image->SaveAs ("rotatedRight.tif", ciEXT);
				}
			else
				Image->SaveAs ("rotatedLeft.tif", ciEXT);
			}
		}
	catch (_com_error &ex)
		{
		dump_com_error(ex);
		cout << "Press Enter to continue" << endl;
		cout.flush();
		getchar();
		}
	CoUninitialize();
	return 0;
}

//============================================================
void dump_com_error( _com_error &e ) 
	{ 
	char sMsg[256];
	if (e.Description().length())
		sprintf (sMsg, "=====> COM error: Code: %08lx  Description: %s" , 
						e.Error(), (LPCSTR)e.Description()); 
	else
		sprintf (sMsg, "=====> COM error: Code: %08lx  Message: %s" , 
						e.Error(), (LPCSTR)e.ErrorMessage()); 
		
	cout << sMsg << endl;	
	OutputDebugString(sMsg); 
	}

bool ReadOneMicr  (char *fileName)
	{
	try 
		{
		_CcMicrReaderPtr Reader;
		HRESULT hr = Reader.CreateInstance(__uuidof(CcMicrReader));
		Reader->Image->Open(_bstr_t(fileName), 1);
		int cnt = Reader->FindMICR();
		if (cnt > 0)
			{
	  	 _CcMicrPtr micr = Reader->MicrLine[1];
			 printf ("MICR Document Type %s\n", (LPCTSTR)(micr->DocumentType));
			 printf ("MICR: %s\n",  (LPCTSTR) (micr->Info->TextRaw));
		   if (micr->AuxOnUs->IsRead)
				 printf ("AuxOnUs: %s\n",  (LPCTSTR) (micr->AuxOnUs->TextANSI));
		   if (micr->EPC->IsRead)
				 printf ("EPC: %s\n",  (LPCTSTR) (micr->EPC->TextANSI));
		   if (micr->Routing->IsRead)
				 printf ("Routing: %s\n",  (LPCTSTR) (micr->Routing->TextANSI));
		   if (micr->RoutingChecksum->IsRead)
				 printf ("RoutingChecksum: %s\n",  (LPCTSTR) (micr->RoutingChecksum->TextANSI));
		   if (micr->OnUs->IsRead)
				 printf ("OnUs: %s\n",  (LPCTSTR) (micr->OnUs->TextANSI));
		   if (micr->Amount->IsRead)
				 printf ("Amount: %s\n",  (LPCTSTR) (micr->Amount->TextANSI));
		   if (micr->Account->IsRead)
				 printf ("Account: %s\n",  (LPCTSTR) (micr->Account->TextANSI));
		   if (micr->CheckNumber->IsRead)
				 printf ("CheckNumber: %s\n",  (LPCTSTR) (micr->CheckNumber->TextANSI));
			}
			return true;
		}
	catch (_com_error &ex)
		{
		printf ("Error: Code: %08lx  Description: %s" , 
						ex.Error(), (LPCSTR)ex.Description()); 
		return false;
		}
	}
	